package eu.cedarsoft.devtools.guice;

import com.google.inject.AbstractModule;
import com.google.inject.Binding;
import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Provider;
import com.google.inject.Scopes;
import com.google.inject.TypeLiteral;
import com.google.inject.name.Names;
import eu.cedarsoft.devtools.DirectorySupport;
import eu.cedarsoft.devtools.MavenBin;
import eu.cedarsoft.devtools.MavenConfiguration;
import eu.cedarsoft.devtools.MavenSupport;
import eu.cedarsoft.utils.configuration.ConfigurationAccess;
import eu.cedarsoft.utils.configuration.ConfigurationManager;
import eu.cedarsoft.utils.configuration.DefaultValueProvider;
import org.apache.commons.configuration.Configuration;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

/**
 * The Maven Module
 */
public class MavenModule extends AbstractModule {
  @Override
  protected void configure() {
    bindConfiguration();

    bindMavenBin();

    bindMavenSupport();

    bindArchetypes();
  }

  private void bindArchetypes() {
    bind( MavenSupport.Archetype.class ).annotatedWith( Names.named( "maven.archetype.0" ) ).toProvider( new Provider<MavenSupport.Archetype>() {
      public MavenSupport.Archetype get() {
        return new MavenSupport.Archetype( "org.apache.maven.archetypes", "maven-archetype-quickstart", "archetype.default.application" );
      }
    } ).in( Scopes.SINGLETON );

    bind( MavenSupport.Archetype.class ).annotatedWith( Names.named( "maven.archetype.1" ) ).toProvider( new Provider<MavenSupport.Archetype>() {
      public MavenSupport.Archetype get() {
        return new MavenSupport.Archetype( "org.apache.maven.archetypes", "maven-archetype-archetype", "archetype.archetype" );
      }
    } ).in( Scopes.SINGLETON );

    bind( MavenSupport.Archetype.class ).annotatedWith( Names.named( "maven.archetype.2" ) ).toProvider( new Provider<MavenSupport.Archetype>() {
      public MavenSupport.Archetype get() {
        return new MavenSupport.Archetype( "org.apache.maven.archetypes", "maven-archetype-site", "archetype.site" );
      }
    } ).in( Scopes.SINGLETON );

    bind( MavenSupport.Archetype.class ).annotatedWith( Names.named( "maven.archetype.3" ) ).toProvider( new Provider<MavenSupport.Archetype>() {
      public MavenSupport.Archetype get() {
        return new MavenSupport.Archetype( "org.apache.maven.archetypes", "maven-archetype-webapp", "archetype.web.application" );
      }
    } ).in( Scopes.SINGLETON );

    bind( MavenSupport.Archetype.class ).annotatedWith( Names.named( "maven.archetype.4" ) ).toProvider( new Provider<MavenSupport.Archetype>() {
      public MavenSupport.Archetype get() {
        return new MavenSupport.Archetype( "org.springframework.richclient", "archetype.spring.rcp", "archetype.spring.rcp", "0.3.0-SNAPSHOT" );
      }
    } ).in( Scopes.SINGLETON );

    bind( MavenSupport.Archetype.class ).annotatedWith( Names.named( "maven.archetype.5" ) ).toProvider( new Provider<MavenSupport.Archetype>() {
      public MavenSupport.Archetype get() {
        return new MavenSupport.Archetype( "eu.cedarsoft.maven", "archetype-spring-rcp-jide", "archetype.spring.rcp.jide", "1.0-SNAPSHOT" );
      }
    } ).in( Scopes.SINGLETON );
  }

  private void bindMavenSupport() {
    bind( MavenSupport.class ).toProvider( new Provider<MavenSupport>() {
      @Inject
      DirectorySupport directorySupport;

      @Inject
      @MavenBin
      String mavenBin;

      @Inject
      Injector injector;

      public MavenSupport get() {
        MavenSupport mavenSupport = new MavenSupport( directorySupport, mavenBin );

        List<MavenSupport.Archetype> archetypes = new ArrayList<MavenSupport.Archetype>();

        List<Binding<MavenSupport.Archetype>> bindings = injector.findBindingsByType( TypeLiteral.get( MavenSupport.Archetype.class ) );
        for ( Binding<MavenSupport.Archetype> binding : bindings ) {
          archetypes.add( binding.getProvider().get() );
        }

        mavenSupport.setArchetypes( archetypes );
        return mavenSupport;
      }
    } ).in( Scopes.SINGLETON );
  }

  private void bindMavenBin() {
    bind( String.class ).annotatedWith( MavenBin.class ).toProvider( new Provider<String>() {
      @Inject
      DefaultValueProvider defaultValueProvider;

      @Inject
      @MavenConfiguration
      Configuration configuration;

      public String get() {
        ConfigurationAccess<String> access = new ConfigurationAccess<String>( configuration, String.class, "maven.bin", defaultValueProvider );
        return access.resolve();
      }
    } );
  }

  private void bindConfiguration() {
    bind( Configuration.class ).annotatedWith( MavenConfiguration.class ).toProvider( new Provider<Configuration>() {
      @Inject
      ConfigurationManager configurationManager;

      @NotNull
      public Configuration get() {
        return configurationManager.getModuleConfiguration( MavenSupport.class );
      }
    } ).in( Scopes.SINGLETON );
  }
}
